@@ -59,9 +59,81 @@ d/dt(blood)     = a*intestine - b*blood
   sink() # nolint
 }
 
+.generateRandomUiFun <- function(fun, args, vals=NULL) {
+  .ret <- as.call(c(
+    quote(`{`),
+    do.call(`c`,
+            lapply(args, function(arg) {
+              .dotArg <- str2lang(paste0(".", arg))
+              .arg <- str2lang(arg)
+              .ret <- bquote({
+                .(.dotArg) <- as.character(substitute(.(.arg)))
+                .tmp <- suppressWarnings(try(force(.(.arg)), silent=TRUE))
+                if (!inherits(.tmp, "try-error")) {
+                  if (is.character(.tmp)) {
+                    .(.dotArg) <- .(.arg)
+                  }
+                }
+              })
+              lapply(seq_along(.ret)[-1], function(i) {
+                .ret[[i]]
+              })
+            })),
+    str2lang(paste0("list(replace=paste0('", fun, "(', ", paste(paste0(".", args), collapse=", ', ', "), ", ')'))"))
+  ))
+  if (is.null(vals)) {
+    .f <- paste0(".", fun, " <- function(", paste(args, collapse=", "), ") ",
+                 paste(deparse(.ret), collapse="\n"),
+                 "\n")
+  } else {
+    .f <- paste0(".", fun, " <- function(", paste0(paste0(args, ifelse(is.na(vals), "", " = "), ifelse(is.na(vals), "", vals)),
+                                                  collapse=", "), ") ",
+                 paste(deparse(.ret), collapse="\n"),
+                 "\n")
+  }
+  .f <- paste0(.f, "\n#'@export\nrxUdfUi.", fun, " <- rxUdfUi.rxpois\n\n")
+  .f
+}
+
+.generateRandomUiFuns <- function() {
+  .lst <- list("rxnorm"=c("mean"=0, "sd"=1),
+               "rxpois"="lambda",
+               "rxt"="df",
+               "rxunif"=c("min"=0, "max"=1),
+               "rxweibull"=c("shape"=NA, "scale"=1),
+               "rxgeom"="prob",
+               "rxbeta"=c("shape1", "shape2"),
+               "rxgamma"=c("shape"=NA, "rate"=1),
+               "rxf"=c("df1","df2"),
+               "rxexp"="rate",
+               "rxchisq"="df",
+               "rxcauchy"=c(location = 0, scale = 1),
+               "rxbinom"=c("size", "prob"))
+  .lst2 <- .lst
+  names(.lst2) <- gsub("rx", "ri", names(.lst2))
+  .lst <- c(.lst, .lst2)
+  paste0("# This file is generated by .generateRandomUiFuns() in build.R\n## nocov start\n",
+         vapply(names(.lst), function(fun) {
+           if (is.null(names(.lst[[fun]]))) {
+             .generateRandomUiFun(fun, .lst[[fun]])
+           } else {
+             .generateRandomUiFun(fun, names(.lst[[fun]]), .lst[[fun]])
+           }
+         }, character(1), USE.NAMES=FALSE),
+         "## nocov end")
+}
+
 
 .rxodeBuildCode <- function() {
   # This builds the code needed for rxode2
+  message("Generate rxode2 random named arguments option")
+
+  .l <- .generateRandomUiFuns()
+  .R <- file(devtools::package_file("R/rxrandomui.R"), "wb")
+  writeLines(.l, .R)
+  close(.R)
+
+  message("done")
   message("Generate grammar include file")
   dparser::mkdparse(devtools::package_file("inst/tran.g"),
                     devtools::package_file("src/"),
