@@ -1225,6 +1225,19 @@ rxSolve <- function(object, params = NULL, events = NULL, inits = NULL,
 #' @export
 rxSolve.function <- function(object, params = NULL, events = NULL, inits = NULL, ...,
                              theta = NULL, eta = NULL, envir=parent.frame()) {
+  rxUdfUiReset()
+  if (rxIs(events, "event.data.frame")) {
+    rxUdfUiData(events)
+  } else if (rxIs(params, "event.data.frame")) {
+    rxUdfUiData(params)
+  } else {
+    stop("Cannot detect an event data frame to use while re-parsing the model",
+         call.=FALSE)
+  }
+  rxUdfUiEst("rxSolve")
+  on.exit({
+    rxUdfUiReset()
+  })
   .udfEnvSet(list(envir, parent.frame(1)))
   .object <- rxode2(object)
   do.call("rxSolve", c(list(object=.object, params = params, events = events, inits = inits),
@@ -1375,6 +1388,27 @@ rxSolve.function <- function(object, params = NULL, events = NULL, inits = NULL,
 #' @export
 rxSolve.rxUi <- function(object, params = NULL, events = NULL, inits = NULL, ...,
                          theta = NULL, eta = NULL, envir=parent.frame()) {
+  rxUdfUiReset()
+  if (isTRUE(object$uiUseData)) {
+    # this needs to be re-parsed
+    if (rxIs(events, "event.data.frame")) {
+      rxUdfUiData(events)
+      rxUdfUiMv(rxModelVars(object))
+    } else if (rxIs(params, "event.data.frame")) {
+      rxUdfUiData(params)
+      rxUdfUiMv(rxModelVars(object))
+    } else {
+      stop("Cannot detect an event data frame to use while re-parsing the model",
+           call.=FALSE)
+    }
+    rxUdfUiEst("rxSolve")
+    on.exit({
+      rxUdfUiReset()
+    })
+    # Now re-parse
+    object <- as.function(object)
+    object <- suppressMessages(rxode2(object))
+  }
   .udfEnvSet(list(object$meta, envir, parent.frame(1)))
   if (inherits(object, "rxUi")) {
     object <- rxUiDecompress(object)
@@ -1423,6 +1457,7 @@ rxSolve.rxode2tos <- rxSolve.rxUi
 #' @export
 rxSolve.nlmixr2FitData <- function(object, params = NULL, events = NULL, inits = NULL, ...,
                                    theta = NULL, eta = NULL, envir=parent.frame()) {
+  rxUdfUiReset()
   .udfEnvSet(list(envir, parent.frame(1)))
   .lst <- .rxSolveFromUi(object, params = params, events = events, inits = inits, ..., theta = theta, eta = eta)
   .rxControl <- .lst[[2]]
@@ -1432,11 +1467,13 @@ rxSolve.nlmixr2FitData <- function(object, params = NULL, events = NULL, inits =
     .oldControl <- get("control", envir=.env)
     assign("control", .rxControl, envir=.env)
     on.exit({
+      rxUdfUiReset()
       assign("control", .oldControl, envir=.env)
     })
   } else {
     assign("control", .rxControl, envir=.env)
     on.exit({
+      rxUdfUiReset()
       rm(list="control", envir=.env)
     })
   }
@@ -1458,8 +1495,10 @@ rxSolve.nlmixr2FitCore <- rxSolve.nlmixr2FitData
 #' @export
 rxSolve.default <- function(object, params = NULL, events = NULL, inits = NULL, ...,
                             theta = NULL, eta = NULL, envir=parent.frame()) {
+  rxUdfUiReset()
   .udfEnvSet(list(envir, parent.frame(1)))
   on.exit({
+    rxUdfUiReset()
     .clearPipe()
     .asFunctionEnv$rx <- NULL
   })
